<template>
    <div class="system-add-user-container">
        <el-dialog :title="title" v-model="isShowUserDialog" width="769px">
            <el-form :model="userForm" size="default" ref="userRef" :rules="userFormRules" label-width="90px">
                <el-row :gutter="35">
                    <el-col :xs="24" :sm="12" :md="12" :lg="12" :xl="12" class="mb25">
                        <el-form-item label="用户名" prop="name">
                            <el-input v-model.trim="userForm.name" placeholder="用户名" clearable></el-input>
                        </el-form-item>
                    </el-col>
                    <el-col :xs="24" :sm="12" :md="12" :lg="12" :xl="12" class="mb25">
                        <el-form-item label="真实姓名" prop="realName">
                            <el-input v-model.trim="userForm.realName" placeholder="真实姓名" clearable></el-input>
                        </el-form-item>
                    </el-col>
                    <el-col :xs="24" :sm="12" :md="12" :lg="12" :xl="12" class="mb25">
                      <el-form-item label="账户密码" prop="pwd">
                        <el-input v-model.trim="userForm.pwd" placeholder="账户密码" type="password" show-password>
                        </el-input>
                      </el-form-item>
                    </el-col>
<!--                    <el-col :xs="24" :sm="12" :md="12" :lg="12" :xl="12" class="mb20" v-if="isAdd">-->
<!--                      <el-form-item label="账户密码" prop="pwd">-->
<!--                        <el-input v-model.trim="userForm.pwd" placeholder="账户密码" type="password" show-password>-->
<!--                        </el-input>-->
<!--                      </el-form-item>-->
<!--                    </el-col>-->
                    <el-col :xs="24" :sm="12" :md="12" :lg="12" :xl="12" class="mb25">
                        <el-form-item label="关联角色" prop="roleIds">
                            <el-select v-model="userForm.roleIds" placeholder="关联角色" clearable class="w100" multiple>
                                <el-option v-for="item in roleData" :key="item.id" :label="item.name" :value="item.id"> </el-option>
                            </el-select>
                        </el-form-item>
                    </el-col>
                    <el-col :xs="24" :sm="12" :md="12" :lg="12" :xl="12" class="mb25">
                        <el-form-item label="部门" prop="depId">
                            <el-cascader :options="departmentData" :props="{ emitPath: false, checkStrictly: true, value: 'id', label: 'depName' }" placeholder="部门" clearable class="w100" v-model="userForm.depId"> </el-cascader>
                        </el-form-item>
                    </el-col>
                    <el-col :xs="24" :sm="12" :md="12" :lg="12" :xl="12" class="mb25">
                        <el-form-item label="手机号" prop="phone">
                            <el-input v-model.trim="userForm.phone" placeholder="手机号" clearable></el-input>
                        </el-form-item>
                    </el-col>
                  <el-col :xs="24" :sm="12" :md="12" :lg="12" :xl="12" class="mb25">
                    <el-form-item label="是否专家" prop="identityStatus">
                      <el-radio-group v-model="userForm.identityStatus">
                        <el-radio :label="0">是</el-radio>
                        <el-radio :label="1">否</el-radio>
                      </el-radio-group>
                    </el-form-item>
                  </el-col>
                  <el-col :xs="24" :sm="12" :md="12" :lg="12" :xl="12" class="mb25" v-if="userForm.identityStatus == 0">
                    <el-form-item label="专家类型" prop="identityIds">
                      <el-select v-model="userForm.identityIds" placeholder="专家类型" clearable class="w100" multiple>
                        <el-option v-for="item in expData" :key="item.id" :label="item.identity" :value="item.id"> </el-option>
                      </el-select>
                    </el-form-item>
                  </el-col>
                    <el-col :xs="24" :sm="12" :md="12" :lg="12" :xl="12" class="mb25">
                      <el-form-item label="证件类型" prop="idType">
                        <el-select v-model="userForm.idType" placeholder="证件类型" clearable class="w100">
                          <el-option :key="1" label="身份证" :value="1"> </el-option>
                        </el-select>
                      </el-form-item>
                    </el-col>
                    <el-col :xs="24" :sm="12" :md="12" :lg="12" :xl="12" class="mb25">
                        <el-form-item label="证件号码" prop="idSerial">
                            <el-input v-model.trim="userForm.idSerial" placeholder="证件号码" clearable></el-input>
                        </el-form-item>
                    </el-col>
                  <el-col :xs="24" :sm="12" :md="12" :lg="12" :xl="12" class="mb25" v-if="userForm.identityStatus == 0">
                    <el-form-item label="资质证书" prop="">
                      <el-upload accept="image/*" :action="uploadUrl" :headers="header" method="post" :data="{module: 'accountPath'}" :on-success="handleAvatarSuccess" :on-exceed="showTip" :on-preview="handlePictureCardPreview" :limit='imgLimit' v-model:file-list="fileList" list-type="picture-card" :before-upload="picSize" :on-remove="handleRemove" :before-remove="beforeRemove">
                        <el-icon><Plus /></el-icon>
                        <template #tip>
                          <div class="el-upload__tip">上传jpg/png图片尺寸小于5M,最多可上传1张</div>
                        </template>
                      </el-upload>
                    </el-form-item>
                  </el-col>
                </el-row>
            </el-form>
            <template #footer>
                <span class="dialog-footer">
                    <el-button @click="isShowUserDialog = !isShowUserDialog" size="default">取 消</el-button>
                    <el-button type="primary" v-throttle @click="onSubmit" size="default">确 定</el-button>
                </span>
            </template>
        </el-dialog>
        <el-dialog v-model="dialogVisible">
          <el-image style="width: 100%; height: 100%" :src="dialogImageUrl"/>
        </el-dialog>
    </div>
</template>

<script lang="ts">
import { reactive, toRefs, onMounted, defineComponent, ref } from 'vue';
import { ElMessageBox, ElMessage } from 'element-plus';
import { userApi } from '/@/api/systemManage/user';
import {checkChineseName, verifyFullName, verifyIdCard, verifyPhone, verifyPwd} from "/@/utils/toolsValidate";
import axios from 'axios';
import Cookies from "js-cookie";
import {useUserInfo} from "/@/stores/userInfo";
import {storeToRefs} from "pinia";
const userInfo = useUserInfo();
const { userInfos } = storeToRefs(userInfo);
import type { UploadProps } from 'element-plus'
// 定义接口来定义对象的类型
interface DeptData {}
interface roleData {}
interface UserState {
    title: string;
    isShowUserDialog: boolean;
    userForm: {
        name: string;
        realName: string;
        roleIds: Array<any>
        depId: number | null;
        phone: string;
        identityStatus:number|null;
        identityIds: Array<any>
        pwd: string;
        idType: number | null;
        idSerial: string;
      qualificationAttId: number | null;
    };
    userFormRules:{},
    departmentData: Array<DeptData>;
    roleData: Array<roleData>;
    expData: Array<roleData>;
    isAdd:boolean
    dialogVisible: Boolean,
    dialogImageUrl: string | null,
  fileList: Array<file>,
  imgLimit: number,
  uploadUrl: string,
  isOverSize: boolean,
  header:{}
}
interface file {
  url: string;
}



export default defineComponent({
    name: 'userDialog',
    setup(props, context) {
        const userRef = ref()
      const validatePwd = (rule: any, value: any, callback: any)=>{
        if(value === '' && state.title === '新增用户'){
          callback(new Error('请输入密码'))
        }else if(value!='' && value != undefined){
          if(!verifyPwd(value)){
            callback(new Error('密码须包含大小写字母、数字、特殊字符，长度不少于8位'))
          }else{
            callback()
          }
        }else {
          callback()
        }
      }
        const state = reactive<UserState>({
            title: '',
            isShowUserDialog: false,
            userForm: {
                name: '', // 账户名称
                realName: '', // 用户昵称
                pwd: '', // 账户密码
                roleIds: [], // 关联角色
                depId: null, // 部门
                phone: '', // 手机号
                identityStatus: null,
                identityIds: [],
                idType: 1,
                idSerial: '',
              qualificationAttId: null
            },
            userFormRules:{
                name: [{ required: true, message: '请填写用户名', trigger: 'blur' }],
                realName: [{ required: true, message: '请填写真实姓名', trigger: 'blur' }],
                roleIds: [{ required: true, message: '请选择用户角色', trigger: 'change' }],
                depId: [{ required: true, message: '请选择部门', trigger: 'change' }],
                phone: [{ required: true, message: '请填写手机号', trigger: 'blur' }],
                identityStatus: [{ required: true, message: '请选择是否为专家', trigger: 'change' }],
                identityIds: [{ required: true, message: '请选择专家类型', trigger: 'change' }],
                idType: [{ required: true, message: '请选择证件类型', trigger: 'blur' }],
                pwd: [{validator: validatePwd, trigger: 'blur' }],
            },
            departmentData: [], // 部门数据
            roleData: [], //角色数据
            expData: [],
            isAdd: true,
          dialogImageUrl: null,
          dialogVisible: false,
          fileList: [],
          imgLimit: 1,
          uploadUrl: import.meta.env.VITE_API_URL + '/attachment/upload/detail',
          isOverSize: false,
          header: {
            uid: Number(userInfos.value.uid),
            tk: Cookies.get('token')
          }
        });


        // 打开弹窗
        const openDialog = (type: string, value: any, departmentList: [], roleList: [],expList: []) => {
            state.isShowUserDialog = true;
            state.departmentData = departmentList;
            state.roleData = roleList;
            state.expData = expList
            if (type === '新增') {
                state.title = '新增用户';
                state.isAdd = true
                state.userForm = {
                    name: '',
                    realName: '',
                    roleIds: [],
                    depId: null,
                    phone: '',
                    identityStatus: null,
                    identityIds: [],
                    idType: 1,
                    idSerial: '',
                    pwd: '',
                  qualificationAttId: null
                };
            } else {
                state.title = '修改用户';
                state.isAdd = false
                state.userForm = JSON.parse(JSON.stringify(value));
                state.userForm.pwd = '';
                state.userForm.roleIds = JSON.parse(JSON.stringify(value)).roles.map(i=>i.roleId)
                state.userForm.identityIds = JSON.parse(JSON.stringify(value)).userIdentities.map(i=>i.userIdentityId)
            }
        };

        // 新增修改
        const onSubmit = async () => {
            userRef.value.validate(async (valid:Boolean) => {
                if(valid){
                    if(checkChineseName(state.userForm.realName) == false){
                      ElMessage({
                        type: 'warning',
                        message: '真实姓名格式有误',
                        duration: 2000
                      });
                      return
                    }
                    if(verifyPhone(state.userForm.phone) == false){
                      ElMessage({
                        type: 'warning',
                        message: '请输入正确的手机号',
                        duration: 2000
                      });
                      return
                    }
                    if(state.userForm.idSerial!=='' && verifyIdCard(state.userForm.idSerial) == false){
                      ElMessage({
                        type: 'warning',
                        message: '请输入正确的证件号码',
                        duration: 2000
                      });
                      return
                    }
                    if(state.userForm.identityStatus == 1){
                      state.userForm.identityIds = []
                      state.userForm.qualificationAttId = null
                    }
                    if (state.title === '新增用户') {
                        let res = await userApi().addUser(state.userForm);
                        if (res.data.code === 100) {
                            ElMessage({
                                type: 'success',
                                message: '用户新增成功',
                                duration: 2000
                            });
                            state.isShowUserDialog = false;
                            context.emit('getUserList');
                        } else {
                            ElMessage({
                                type: 'warning',
                                message: res.data.msg
                            });
                        }
                    } else {
                        const {name, realName, roleIds, depId, phone, idType, idSerial, id, identityStatus, identityIds, qualificationAttId, pwd} = state.userForm
                        const data = {name, realName, roleIds, depId, phone, idType, idSerial,id, identityStatus, identityIds, qualificationAttId, pwd}
                        let res = await userApi().modUser(data);
                        if (res.data.code === 100) {
                            ElMessage({
                                type: 'success',
                                message: '用户修改成功',
                                duration: 2000
                            });
                            state.isShowUserDialog = false;
                            context.emit('getUserList');
                        } else {
                            ElMessage({
                                type: 'warning',
                                message: res.data.msg
                            });
                        }
                    }
                }else{
                    ElMessage({
                        type:'warning',
                        message:'请完善基本信息'
                    })
                }
            })

        };

        // 图片上传
        const showTip =()=>{
          ElMessage({
            type: 'warning',
            message: '超出文件上传数量'
          });
        }

      const picSize = async (rawFile: any) => {
        if(rawFile.size / 1024 / 1024 > 5){
          ElMessage({
            type: 'warning',
            message: '文件大小不能超过5M。'
          });
          return false
        }
      };

      const handlePictureCardPreview = (uploadFile: { url: string }) => {
        state.dialogImageUrl = uploadFile.url!;
        state.dialogVisible = true;
      };

      const upload = async (params: any) => {
        // const formData = new FormData();
        // formData.append('file', state.fileList[0].raw);
        let reader = new FileReader();
        reader.readAsArrayBuffer(params.file);

        reader.onload = async () => {
          axios.post(state.uploadUrl, reader.result, {
                headers: { uid: Number(userInfos.value.uid),tk: Cookies.get('token') }
              }).then(res => {
                if(res.data.code === 100){
                  console.log(res.data.data)
                }
              });
        };
      };

      const handleAvatarSuccess: UploadProps['onSuccess'] = (
          res,
          uploadFile
      ) => {
        if(res.code == 100){
          state.userForm.qualificationAttId = res.data.id
        }
      }

      // const beforeRemove = (file: {}, fileList: []) => {
      //   const result = new Promise((resolve, reject) => {
      //     if(!state.isOverSize){
      //       ElMessageBox.confirm('此操作将删除该图片, 是否继续?', '提示', {
      //         confirmButtonText: '确定',
      //         cancelButtonText: '取消',
      //         type: 'warning'
      //       })
      //           .then(() => {
      //             // console.log(state.workDetail.gbPath,'path')
      //             const list = JSON.parse(JSON.stringify(state.form.workDetail.gbPath))
      //             fileList.map((item,index)=>{
      //               if(item.uid === file.uid){
      //                 fileList.splice(index,1)
      //                 state.form.workDetail.gbPath.splice(index,1)
      //                 // 请求删除接口
      //                 deletePic(false,list[index])
      //               }
      //             })
      //           })
      //           .catch(() => {
      //             reject(false);
      //           });
      //     }else{
      //       const list = JSON.parse(JSON.stringify(state.form.workDetail.gbPath))
      //       fileList.map((item,index)=>{
      //         if(item.uid === file.uid){
      //           fileList.splice(index,1)
      //           state.form.workDetail.gbPath.splice(index,1)
      //           deletePic(true,list[index])
      //         }
      //       })
      //       state.isOverSize = false
      //     }
      //   });
      //   return result;
      // };

        // 页面加载时
        onMounted(() => {

        });
        return {
            userRef,
            openDialog,
          picSize,
            showTip,
          handlePictureCardPreview,
          upload,
          handleAvatarSuccess,
            onSubmit,
            ...toRefs(state)
        };
    }
});
</script>
